/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client;

import java.text.MessageFormat;

import javax.security.auth.login.LoginContext;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.client.AgreementFactoryRegistryClient;
import org.ogf.graap.wsag.api.configuration.WSAG4JConfiguration;
import org.ogf.graap.wsag.client.impl.AgreementFactoryRegistryBuilder;
import org.ogf.graap.wsag.client.impl.DefaultAgreementFactoryRegistryBuilder;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/*
 * AgreementFactoryLocator
 */

/**
 * The AgreementFactoryLocator returns a new agreement factory instance.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class AgreementFactoryRegistryLocator
{

    private static AgreementFactoryRegistryBuilder creator = null;

    private static final Logger LOG = Logger.getLogger( AgreementFactoryRegistryLocator.class );

    /**
     * System property key to lookup {@link AgreementFactoryRegistryBuilder} implementation
     */
    public static final String REGISTRY_LOCATOR = "org.ogf.graap.wsag.client.AgreementFactoryRegistryLocator";

    /**
     * Creates a new {@link AgreementFactoryRegistryClient} for the given endpoint. This method is only used
     * if server side security is disabled.
     * 
     * @param epr
     *            the agreement factory registry endpoint
     * 
     * @return the new {@link AgreementFactoryRegistryClient}
     * 
     * @throws Exception
     *             indicates an error while querying the factory clients from the registry
     */
    public static final synchronized AgreementFactoryRegistryClient getFactoryRegistry( EndpointReferenceType epr )
        throws Exception
    {
        return getFactoryRegistry( epr, (LoginContext) null );
    }

    /**
     * Creates a new {@link AgreementFactoryRegistryClient} for the given endpoint and with the given
     * {@link LoginContext}. This method is used if server side security is enabled (default).
     * 
     * @param epr
     *            the agreement factory registry endpoint
     * 
     * @param context
     *            the login context to use
     * 
     * @return the new {@link AgreementFactoryRegistryClient}
     * 
     * @throws Exception
     *             indicates an error while querying the factory clients from the registry
     */
    public static final synchronized AgreementFactoryRegistryClient getFactoryRegistry( EndpointReferenceType epr,
                                                                                        LoginContext context )
        throws Exception
    {
        if ( creator == null )
        {

            //
            // First check if the registry locator implementation is provided via system properties.
            //
            if ( System.getProperties().containsKey( REGISTRY_LOCATOR ) )
            {
                String implClass = System.getProperties().getProperty( REGISTRY_LOCATOR );
                try
                {
                    if ( LOG.isDebugEnabled() )
                    {
                        LOG.debug( "AgreementFactoryRegistryLocator implementation (" + implClass
                            + ") specified in system properties." );
                    }

                    @SuppressWarnings( "unchecked" )
                    Class<AgreementFactoryRegistryBuilder> clazz =
                        (Class<AgreementFactoryRegistryBuilder>) Class.forName( implClass );
                    creator = (AgreementFactoryRegistryBuilder) clazz.newInstance();
                }
                catch ( ClassNotFoundException e )
                {
                    String message =
                        "AgreementFactoryRegistryBuilder implementation ({0}) not found. "
                            + "Load implementation specified in {1}.";
                    LOG.warn( MessageFormat.format( message, new Object[] { implClass,
                        WsagConstants.WSAG4J_CLIENT_CONFIG_FILE } ) );
                }
                catch ( ClassCastException e )
                {
                    String message =
                        "AgreementFactoryRegistryBuilder implementation ({0}) does not implement the interface {1}. "
                            + "Implementation was specified in {2}.";
                    LOG.warn( MessageFormat.format( message, new Object[] { implClass,
                        AgreementFactoryRegistryBuilder.class.getName(),
                        WsagConstants.WSAG4J_CLIENT_CONFIG_FILE } ) );
                }

            }

            //
            // if no implementation was loaded, try the config file
            //
            if ( creator == null )
            {
                Class<AgreementFactoryRegistryBuilder> interfaceDef = AgreementFactoryRegistryBuilder.class;
                String wsag4jClientConfigFile = WsagConstants.WSAG4J_CLIENT_CONFIG_FILE;
                String defaultImpl = DefaultAgreementFactoryRegistryBuilder.class.getName();

                creator =
                    (AgreementFactoryRegistryBuilder) WSAG4JConfiguration.findImplementation( interfaceDef,
                                                                                              wsag4jClientConfigFile,
                                                                                              defaultImpl );
            }
        }
        return creator.newInstance( epr, context );
    }
}
