/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.impl;

import java.util.Properties;

import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.client.AgreementClient;
import org.ogf.graap.wsag.api.client.AgreementFactoryClient;
import org.ogf.graap.wsag.api.client.AgreementFactoryService;
import org.ogf.graap.wsag.api.client.AgreementRegistryService;
import org.ogf.graap.wsag.api.client.NegotiationClient;
import org.ogf.graap.wsag.api.client.WsClient;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * A client side facade to an agreement factory service and the associated agreement registry.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class AgreementFactoryFacade
    implements AgreementFactoryClient
{

    //
    // client for invoking WS-Agreement factory methods
    //
    private final AgreementFactoryService factoryClient;

    //
    // client for invoking WSSRF service group methods
    //
    private final AgreementRegistryService registryClient;

    //
    // client properties to store security processing results
    //
    private Properties properties = new Properties();

    //
    // client security properties
    //
    private final ISecurityProperties securityProperties;

    /**
     * Creates a new instance of the facade.
     * 
     * @param factoryClient
     *            the factory client instance
     * 
     * @param registryClient
     *            the registry client instance
     * 
     * @param securityProperties
     *            the security properties to use
     */
    public AgreementFactoryFacade( AgreementFactoryService factoryClient,
                                   AgreementRegistryService registryClient,
                                   ISecurityProperties securityProperties )
    {
        this.factoryClient = factoryClient;
        this.registryClient = registryClient;
        this.securityProperties = securityProperties;

        factoryClient.getWebServiceClient().setProperties( properties );
        registryClient.getWebServiceClient().setProperties( properties );
    }

    /**
     * Returns the templates supported by this factory.
     * 
     * {@inheritDoc}
     */
    public AgreementTemplateType[] getTemplates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return factoryClient.getTemplates();
    }

    /**
     * Creates a new agreement instance.
     * 
     * @param offer
     *            the offer to create the agreement for
     * 
     * @return the created agreement
     * 
     * @throws AgreementFactoryException
     *             the agreement offer was rejected by the factory
     * 
     * @throws ResourceUnavailableException
     *             the requested agreement factory resource is unavailable
     * 
     * @throws ResourceUnknownException
     *             the requested agreement factory resource is unknown
     */
    public AgreementClient createAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return factoryClient.createAgreement( offer );
    }

    /**
     * Creates a new pending agreement instance.
     * 
     * {@inheritDoc}
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return factoryClient.createPendingAgreement( offer );
    }

    /**
     * Creates a new pending agreement instance.
     * 
     * {@inheritDoc}
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer, EndpointReferenceType acceptanceEPR )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return factoryClient.createPendingAgreement( offer, acceptanceEPR );
    }

    /**
     * Initiates a Negotiation.
     * 
     * {@inheritDoc}
     */
    public NegotiationClient initiateNegotiation( NegotiationContextType context )
        throws NegotiationFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return factoryClient.initiateNegotiation( context );
    }

    /**
     * {@inheritDoc}
     */
    public String getResourceId() throws ResourceUnknownException, ResourceUnavailableException
    {
        return factoryClient.getResourceId();
    }

    /**
     * 
     * @return a template with the given name and id, or null if no appropriate template was found.
     * 
     *         {@inheritDoc}
     */
    public AgreementTemplateType getTemplate( String name, String id )
        throws ResourceUnknownException, ResourceUnavailableException
    {
        if ( name == null )
        {
            return null;
        }

        id = ( id != null ) ? id : "";

        AgreementTemplateType[] templates = getTemplates();

        for ( int i = 0; i < templates.length; i++ )
        {
            if ( name.equalsIgnoreCase( templates[i].getName() ) )
            {

                String actualId =
                    ( templates[i].getTemplateId() != null ) ? templates[i].getTemplateId() : "";

                if ( id.equalsIgnoreCase( actualId ) )
                {
                    return templates[i];
                }
            }
        }

        return null;
    }

    /**
     * Creates s copy of this AgreementFactoryClient.
     * 
     * @return the cloned agreement factory client
     * 
     * @throws CloneNotSupportedException
     *             this particular client implementation does not support cloning
     */
    @Override
    public AgreementFactoryClient clone() throws CloneNotSupportedException
    {
        return new AgreementFactoryFacade( factoryClient, registryClient, securityProperties.clone() );
    }

    /**
     * Lists all agreements of this factory.
     * 
     * @return all agreement clients known by this factory
     * 
     * @throws ResourceUnavailableException
     *             the requested agreement factory resource is unavailable
     * 
     * @throws ResourceUnknownException
     *             the requested agreement factory resource is unknown
     */
    public AgreementClient[] listAgreements() throws ResourceUnknownException, ResourceUnavailableException
    {
        return registryClient.listAgreements();
    }

    /**
     * Turns trace on/off.
     * 
     * @param trace
     *            turns trace on/off.
     */
    public void setTrace( boolean trace )
    {
        factoryClient.getWebServiceClient().setTrace( trace );
        registryClient.getWebServiceClient().setTrace( trace );
    }

    /**
     * @return the trace level
     */
    public boolean isUsingTrace()
    {
        return factoryClient.getWebServiceClient().isUsingTrace()
            && registryClient.getWebServiceClient().isUsingTrace();
    }

    /**
     * @return Returns the properties.
     */
    public Properties getProperties()
    {
        return properties;
    }

    /**
     * @param properties
     *            The properties to set.
     */
    public void setProperties( Properties properties )
    {
        this.properties = properties;
        factoryClient.getWebServiceClient().setProperties( properties );
        registryClient.getWebServiceClient().setProperties( properties );
    }

    /**
     * @return Returns the endpoint reference for the service.
     */
    public EndpointReferenceType getEndpoint()
    {
        return factoryClient.getWebServiceClient().getEndpoint();
    }

    /**
     * @return the securityProperties
     */
    public ISecurityProperties getSecurityProperties()
    {
        return securityProperties;
    }

    /**
     * @return the webservice client.
     */
    public WsClient getWebServiceClient()
    {
        return this;
    }

    /**
     * @return Returns the agreement factory client.
     */
    public AgreementFactoryService getAgreementFactoryClient()
    {
        return factoryClient;
    }

    /**
     * @return Returns the agreement registry client.
     */
    public AgreementRegistryService getAgreementRegistryClient()
    {
        return registryClient;
    }
}
