/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.impl;

import java.util.Properties;

import org.ogf.graap.wsag.api.client.AgreementClient;
import org.ogf.graap.wsag.api.client.AgreementService;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.schemas.graap.wsAgreement.AgreementContextType;
import org.ogf.schemas.graap.wsAgreement.AgreementStateType;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermStateType;
import org.ogf.schemas.graap.wsAgreement.ServiceTermStateType;
import org.ogf.schemas.graap.wsAgreement.TermTreeType;
import org.ogf.schemas.graap.wsAgreement.TerminateInputType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * AgreementImpl
 * 
 * @author Oliver Waeldrich
 * 
 */
public abstract class AgreementImpl implements AgreementClient
{

    private AgreementService agreementClient;

    /**
     * Creates an agreement client for the given agreement service implementation.
     * 
     * @param agreementClient
     *            the agreement service client
     */
    protected AgreementImpl( AgreementService agreementClient )
    {
        this.agreementClient = agreementClient;
    }

    /**
     * {@inheritDoc}
     */
    public abstract AgreementClient clone() throws CloneNotSupportedException;

    /**
     * {@inheritDoc}
     */
    public AgreementContextType getContext() throws ResourceUnknownException, ResourceUnavailableException
    {
        return agreementClient.getContext();
    }

    /**
     * {@inheritDoc}
     */
    public GuaranteeTermStateType[] getGuaranteeTermStates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return agreementClient.getGuaranteeTermStates();
    }

    /**
     * {@inheritDoc}
     */
    public GuaranteeTermStateType getGuaranteeTermState( String name )
        throws ResourceUnknownException, ResourceUnavailableException
    {

        GuaranteeTermStateType[] states = getGuaranteeTermStates();
        for ( int i = 0; i < states.length; i++ )
        {
            GuaranteeTermStateType state = states[i];
            if ( name.equals( state.getTermName() ) )
            {
                return state;
            }
        }

        return null;
    }

    /**
     * {@inheritDoc}
     */
    public String getAgreementId() throws ResourceUnknownException, ResourceUnavailableException
    {
        return agreementClient.getAgreementId();
    }

    /**
     * {@inheritDoc}
     */
    public String getName() throws ResourceUnknownException, ResourceUnavailableException
    {
        return agreementClient.getName();
    }

    /**
     * {@inheritDoc}
     */
    public ServiceTermStateType[] getServiceTermStates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return agreementClient.getServiceTermStates();
    }

    /**
     * {@inheritDoc}
     */
    public ServiceTermStateType getServiceTermState( String name )
        throws ResourceUnknownException, ResourceUnavailableException
    {

        ServiceTermStateType[] states = getServiceTermStates();
        for ( int i = 0; i < states.length; i++ )
        {
            ServiceTermStateType state = states[i];
            if ( name.equals( state.getTermName() ) )
            {
                return state;
            }
        }

        return null;
    }

    /**
     * {@inheritDoc}
     */
    public AgreementStateType getState() throws ResourceUnknownException, ResourceUnavailableException
    {
        return agreementClient.getState();
    }

    /**
     * {@inheritDoc}
     */
    public TermTreeType getTerms() throws ResourceUnknownException, ResourceUnavailableException
    {
        return agreementClient.getTerms();
    }

    /**
     * {@inheritDoc}
     */
    public void terminate() throws ResourceUnknownException, ResourceUnavailableException
    {
        terminate( TerminateInputType.Factory.newInstance() );
    }

    /**
     * {@inheritDoc}
     */
    public void terminate( TerminateInputType reason )
        throws ResourceUnknownException, ResourceUnavailableException
    {
        agreementClient.terminate( reason );
    }

    /**
     * {@inheritDoc}
     */
    public void destroy() throws ResourceUnknownException, ResourceUnavailableException
    {
        agreementClient.destroy();
    }

    /**
     * {@inheritDoc}
     */
    public EndpointReferenceType getEndpoint()
    {
        return agreementClient.getWebServiceClient().getEndpoint();
    }

    /**
     * {@inheritDoc}
     */
    public Properties getProperties()
    {
        return agreementClient.getWebServiceClient().getProperties();
    }

    /**
     * {@inheritDoc}
     */
    public void setProperties( Properties properties )
    {
        agreementClient.getWebServiceClient().setProperties( properties );
    }

    /**
     * {@inheritDoc}
     */
    public boolean isUsingTrace()
    {
        return agreementClient.getWebServiceClient().isUsingTrace();
    }

    /**
     * {@inheritDoc}
     */
    public void setTrace( boolean trace )
    {
        agreementClient.getWebServiceClient().setTrace( trace );
    }

    /**
     * {@inheritDoc}
     * 
     * @return the security properties for this client
     * @see org.ogf.graap.wsag.client.remote.WsrfResourceClient#getSecurityProperties()
     */
    public ISecurityProperties getSecurityProperties()
    {
        return agreementClient.getWebServiceClient().getSecurityProperties();
    }

}
