/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.impl;

import java.util.Properties;

import org.ogf.graap.wsag.api.client.NegotiationClient;
import org.ogf.graap.wsag.api.client.NegotiationService;
import org.ogf.graap.wsag.api.exceptions.NegotiationException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * NegotiationClientImpl
 * 
 * @author hrasheed
 * 
 */
public abstract class NegotiationClientImpl implements NegotiationClient
{

    private NegotiationService negotiationClient;

    /**
     * Creates a new negotiation client for the given negotiation service implementation.
     * 
     * @param negotiationClient
     *            the negotiation service client implemenation
     */
    protected NegotiationClientImpl( NegotiationService negotiationClient )
    {
        this.negotiationClient = negotiationClient;
    }

    /**
     * {@inheritDoc}
     */
    public abstract NegotiationClient clone() throws CloneNotSupportedException;

    /**
     * {@inheritDoc}
     */
    public NegotiationContextType getNegotiationContext()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return negotiationClient.getNegotiationContext();
    }

    /**
     * {@inheritDoc}
     */
    public AgreementTemplateType[] getNegotiableTemplates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return negotiationClient.getNegotiableTemplates();
    }

    /**
     * {@inheritDoc}
     */
    public NegotiationOfferType[] getNegotiationOffers()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return negotiationClient.getNegotiationOffers();
    }

    /**
     * {@inheritDoc}
     */
    public NegotiationOfferType[] negotiate( NegotiationOfferType[] quotes )
        throws NegotiationException, ResourceUnknownException, ResourceUnavailableException
    {
        return negotiationClient.negotiate( quotes );
    }

    /**
     * {@inheritDoc}
     */
    public void advertise( NegotiationOfferType[] quotes )
        throws NegotiationException, ResourceUnknownException, ResourceUnavailableException
    {
        negotiationClient.advertise( quotes );
    }

    /**
     * {@inheritDoc}
     */
    public NegotiationOfferType getNegotiationOffer( String offerID )
        throws ResourceUnknownException, ResourceUnavailableException
    {
        NegotiationOfferType[] offers = getNegotiationOffers();
        for ( int i = 0; i < offers.length; i++ )
        {
            NegotiationOfferType offer = offers[i];
            if ( offerID.equals( offer.getOfferId() ) )
            {
                return offer;
            }
        }

        throw new ResourceUnavailableException( "no negotiationoffer found for offer id: " + offerID );
    }

    /**
     * {@inheritDoc}
     */
    public void terminate() throws ResourceUnknownException, ResourceUnavailableException
    {
        negotiationClient.terminate();
    }

    /**
     * {@inheritDoc}
     */
    public void destroy() throws ResourceUnknownException, ResourceUnavailableException
    {
        negotiationClient.destroy();
    }

    /**
     * {@inheritDoc}
     */
    public EndpointReferenceType getEndpoint()
    {
        return negotiationClient.getWebServiceClient().getEndpoint();
    }

    /**
     * {@inheritDoc}
     */
    public Properties getProperties()
    {
        return negotiationClient.getWebServiceClient().getProperties();
    }

    /**
     * {@inheritDoc}
     */
    public void setProperties( Properties properties )
    {
        negotiationClient.getWebServiceClient().setProperties( properties );
    }

    /**
     * {@inheritDoc}
     */
    public boolean isUsingTrace()
    {
        return negotiationClient.getWebServiceClient().isUsingTrace();
    }

    /**
     * {@inheritDoc}
     */
    public void setTrace( boolean trace )
    {
        negotiationClient.getWebServiceClient().setTrace( trace );
    }

    /**
     * {@inheritDoc}
     * 
     * @return the security properties for this client
     * @see org.ogf.graap.wsag.client.remote.WsrfResourceClient#getSecurityProperties()
     */
    public ISecurityProperties getSecurityProperties()
    {
        return negotiationClient.getWebServiceClient().getSecurityProperties();
    }

}
