/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.local;

import java.util.Properties;

import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementFactory;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.client.AgreementClient;
import org.ogf.graap.wsag.api.client.AgreementFactoryClient;
import org.ogf.graap.wsag.api.client.NegotiationClient;
import org.ogf.graap.wsag.api.client.WsClient;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.graap.wsag.api.security.SecurityProperties;
import org.ogf.graap.wsag.client.impl.AgreementFactoryFacade;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * An implementation of the Client API for local invocation of an agreement factory.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class LocalAgreementFactoryClientImpl
    implements AgreementFactoryClient
{

    private final Agreement[] agreements = new Agreement[0];

    private final AgreementFactory factory;

    private final AgreementFactoryFacade facade;

    /**
     * Creates a client for local invocation of an agreement factory.
     * 
     * @param factory
     *            the factory object
     */
    public LocalAgreementFactoryClientImpl( AgreementFactory factory )
    {
        this.factory = factory;
        this.facade =
            new AgreementFactoryFacade( new LocalAgreementFactoryServiceImpl( factory ),
                new LocalAgreementFactoryRegistryImpl( agreements ), new SecurityProperties( null ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AgreementFactoryClient clone() throws CloneNotSupportedException
    {
        return new LocalAgreementFactoryClientImpl( factory );
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.api.client.AgreementFactoryClient#getWebServiceClient()
     */
    public WsClient getWebServiceClient()
    {
        return facade.getWebServiceClient();
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient createAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return facade.createAgreement( offer );
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return facade.createPendingAgreement( offer );
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer, EndpointReferenceType acceptanceEPR )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return facade.createPendingAgreement( offer, acceptanceEPR );
    }

    /**
     * {@inheritDoc}
     */
    public NegotiationClient initiateNegotiation( NegotiationContextType context )
        throws NegotiationFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return facade.initiateNegotiation( context );
    }

    /**
     * {@inheritDoc}
     */
    public EndpointReferenceType getEndpoint()
    {
        return facade.getEndpoint();
    }

    /**
     * {@inheritDoc}
     */
    public Properties getProperties()
    {
        return facade.getProperties();
    }

    /**
     * {@inheritDoc}
     */
    public String getResourceId() throws ResourceUnknownException, ResourceUnavailableException
    {
        return facade.getResourceId();
    }

    /**
     * {@inheritDoc}
     */
    public ISecurityProperties getSecurityProperties()
    {
        return facade.getSecurityProperties();
    }

    /**
     * {@inheritDoc}
     */
    public AgreementTemplateType getTemplate( String name, String id )
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return facade.getTemplate( name, id );
    }

    /**
     * {@inheritDoc}
     */
    public AgreementTemplateType[] getTemplates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return facade.getTemplates();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isUsingTrace()
    {
        return facade.isUsingTrace();
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient[] listAgreements() throws ResourceUnknownException, ResourceUnavailableException
    {
        return facade.listAgreements();
    }

    /**
     * {@inheritDoc}
     */
    public void setProperties( Properties properties )
    {
        facade.setProperties( properties );
    }

    /**
     * {@inheritDoc}
     */
    public void setTrace( boolean trace )
    {
        facade.setTrace( trace );
    }

}
