/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.local;

import java.util.HashMap;
import java.util.Map;

import org.apache.xmlbeans.XmlObject;
import org.ogf.graap.wsag.api.AgreementFactory;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.client.AgreementClient;
import org.ogf.graap.wsag.api.client.AgreementFactoryService;
import org.ogf.graap.wsag.api.client.NegotiationClient;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * Client implementation of an {@link AgreementFactoryService} for local access.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class LocalAgreementFactoryServiceImpl extends LocalWsClient
    implements AgreementFactoryService
{

    private final AgreementFactory factory;

    /**
     * Creates a client instance for the given agreement factory object.
     * 
     * @param factory
     *            the factory to use
     */
    public LocalAgreementFactoryServiceImpl( AgreementFactory factory )
    {
        this.factory = factory;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.api.client.AgreementFactoryService#createAgreement(org.ogf.graap.wsag.api.AgreementOffer)
     */
    public AgreementClient createAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return new LocalAgreementClientImpl( factory.createAgreement( offer ) );
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.api.client.AgreementFactoryService#createAgreement(org.ogf.graap.wsag.api.AgreementOffer)
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return new LocalAgreementClientImpl( factory.createAgreement( offer ) );
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.api.client.AgreementFactoryService#createAgreement(org.ogf.graap.wsag.api.AgreementOffer)
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer, EndpointReferenceType acceptanceEPR )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        //
        // not supported locally
        //
        return new LocalAgreementClientImpl( factory.createAgreement( offer ) );
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.api.client.AgreementFactoryService#getTemplates()
     */
    public AgreementTemplateType[] getTemplates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return factory.getTemplates();
    }

    /**
     * {@inheritDoc}
     */
    public NegotiationClient initiateNegotiation( NegotiationContextType context )
        throws NegotiationFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        XmlObject[] criticalExtensions = new XmlObject[0];
        XmlObject[] nonCriticalExtensions = new XmlObject[0];
        Map<String, Object> environment = new HashMap<String, Object>();
        return new LocalNegotiationClientImpl( factory.initiateNegotiation( context, criticalExtensions,
            nonCriticalExtensions, environment ) );
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.api.client.WSDMResource#getResourceId()
     */
    public String getResourceId() throws ResourceUnknownException, ResourceUnavailableException
    {
        return "local_id";
    }

}
