/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.remote;

import java.util.Properties;

import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.client.AgreementClient;
import org.ogf.graap.wsag.api.client.AgreementFactoryClient;
import org.ogf.graap.wsag.api.client.NegotiationClient;
import org.ogf.graap.wsag.api.client.WsClient;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.graap.wsag.client.impl.AgreementFactoryFacade;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.w3.x2005.x08.addressing.EndpointReferenceDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * RemoteAgreementFactoryClientImpl
 * 
 * @author Oliver Waeldrich
 * 
 */
public class RemoteAgreementFactoryClientImpl
    implements AgreementFactoryClient
{

    private AgreementFactoryFacade facade;

    /**
     * Creates a new agreement factory client for a given endpoint using the given security properties.
     * 
     * @param epr
     *            the agreement factory endpoint
     * 
     * @param securityProperties
     *            the client security properties
     */
    public RemoteAgreementFactoryClientImpl( EndpointReferenceType epr, ISecurityProperties securityProperties )
    {

        try
        {
            //
            // generate the endpoint references for the registry service
            //
            String factoryURI = epr.getAddress().getStringValue();
            String registryURI =
                factoryURI.substring( 0, factoryURI.indexOf( WsagConstants.AGREEMENT_FACTORY_SERVICE_URI ) )
                    + WsagConstants.AGREEMENT_REGISTRY_SERVICE_URI;

            EndpointReferenceDocument registryEPR = EndpointReferenceDocument.Factory.newInstance();
            registryEPR.addNewEndpointReference().addNewAddress().setStringValue( registryURI );
            registryEPR.getEndpointReference().addNewReferenceParameters().set( epr.getReferenceParameters() );

            //
            // initialize the service clients
            //
            RemoteAgreementFactoryServiceImpl factoryClient =
                new RemoteAgreementFactoryServiceImpl( epr, securityProperties );
            RemoteAgreementRegistryServiceImpl registryClient =
                new RemoteAgreementRegistryServiceImpl( registryEPR.getEndpointReference(),
                    securityProperties );

            facade = new AgreementFactoryFacade( factoryClient, registryClient, securityProperties );
        }
        catch ( Exception e )
        {
            throw new RuntimeException( e );
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AgreementFactoryClient clone() throws CloneNotSupportedException
    {
        return new RemoteAgreementFactoryClientImpl( getEndpoint(), getSecurityProperties().clone() );
    }

    /**
     * {@inheritDoc}
     */
    public WsClient getWebServiceClient()
    {
        return facade.getWebServiceClient();
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient createAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return facade.createAgreement( offer );
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return facade.createPendingAgreement( offer );
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer, EndpointReferenceType acceptanceEPR )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return facade.createPendingAgreement( offer, acceptanceEPR );
    }

    /**
     * {@inheritDoc}
     */
    public EndpointReferenceType getEndpoint()
    {
        return facade.getEndpoint();
    }

    /**
     * {@inheritDoc}
     */
    public Properties getProperties()
    {
        return facade.getProperties();
    }

    /**
     * {@inheritDoc}
     */
    public String getResourceId() throws ResourceUnknownException, ResourceUnavailableException
    {
        return facade.getResourceId();
    }

    /**
     * {@inheritDoc}
     */
    public ISecurityProperties getSecurityProperties()
    {
        return facade.getSecurityProperties();
    }

    /**
     * @param name
     *            the requested template name
     * 
     * @param id
     *            the requested template id
     * 
     * @return the template with the specified name and id. If no template exists with these properties, null
     *         is returned.
     * 
     * @throws ResourceUnknownException
     *             the remote resource is unknown
     * 
     * @throws ResourceUnavailableException
     *             the remote resource is unavailable
     * 
     * @see org.ogf.graap.wsag.client.impl.AgreementFactoryFacade#getTemplate(String, String)
     */
    public AgreementTemplateType getTemplate( String name, String id )
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return facade.getTemplate( name, id );
    }

    /**
     * {@inheritDoc}
     */
    public AgreementTemplateType[] getTemplates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        return facade.getTemplates();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isUsingTrace()
    {
        return facade.isUsingTrace();
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient[] listAgreements() throws ResourceUnknownException, ResourceUnavailableException
    {
        return facade.listAgreements();
    }

    /**
     * {@inheritDoc}
     */
    public void setProperties( Properties properties )
    {
        facade.setProperties( properties );
    }

    /**
     * {@inheritDoc}
     */
    public void setTrace( boolean trace )
    {
        facade.setTrace( trace );
    }

    /**
     * Initiates a new negotiation process with the given context.
     * 
     * @param context
     *            the negotiation context
     * 
     * @return a client to the new created negotiation process
     * 
     * @throws NegotiationFactoryException
     *             The negotiation process was not initiated.
     * 
     * @throws ResourceUnknownException
     *             the remote resource is unknown
     * 
     * @throws ResourceUnavailableException
     *             the remote resource is unavailable
     * 
     * @see org.ogf.graap.wsag.client.impl.AgreementFactoryFacade#initiateNegotiation(NegotiationContextType)
     */
    public NegotiationClient initiateNegotiation( NegotiationContextType context )
        throws NegotiationFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return facade.initiateNegotiation( context );
    }

}
