/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.remote;

import org.apache.log4j.Logger;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.resource.sg.WssgConstants;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlOptions;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.client.AgreementFactoryClient;
import org.ogf.graap.wsag.api.client.AgreementFactoryRegistryClient;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3c.dom.Element;

/**
 * Default implementation of an agreement factory client.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class RemoteAgreementFactoryRegistryServiceImpl extends WsrfResourceClient
    implements AgreementFactoryRegistryClient
{

    private static Logger log = Logger.getLogger( RemoteAgreementFactoryRegistryServiceImpl.class );

    private static EndpointReferenceType createRegistryEPR( EndpointReferenceType epr )
    {
        String address = epr.getAddress().getStringValue();

        //
        // BUG 79: if the address ends with "/", we need to remove it
        //
        while ( address.endsWith( "/" ) )
        {
            address = address.substring( 0, address.length() - 1 );
        }

        epr.getAddress().setStringValue( address + WsagConstants.AGREEMENT_FACTORY_REGISTRY_SERVICE_URI );
        return epr;
    }

    /**
     * Instantiates a new agreement factory client for the given endpoint reference.
     * 
     * @param epr
     *            the agreement factory EPR
     * 
     * @param securityProperties
     *            the security properties
     */
    public RemoteAgreementFactoryRegistryServiceImpl( EndpointReferenceType epr,
                                                      ISecurityProperties securityProperties )
    {
        super( createRegistryEPR( epr ), securityProperties );
    }

    /**
     * {@inheritDoc}
     */
    public AgreementFactoryClient[] listAgreementFactories()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        try
        {
            Element[] entries = getResourceProperty( WssgConstants.ENTRY_QNAME );
            AgreementFactoryClient[] agreements = new AgreementFactoryClient[entries.length];

            for ( int n = 0; n < entries.length; ++n )
            {
                //
                // convert ws-sg entry XML -> EPRs -> clients
                //
                Element eprXML = XmlUtils.getElement( entries[n], WssgConstants.MEMBER_SERVICE_EPR_QNAME );

                EndpointReferenceType dest =
                    EndpointReferenceType.Factory.parse( eprXML,
                                                         new XmlOptions().setLoadReplaceDocumentElement( null ) );

                //
                // create a new agreement instance with the current properties
                //
                agreements[n] = new RemoteAgreementFactoryClientImpl( dest, getSecurityProperties().clone() );
                agreements[n].setTrace( isUsingTrace() );
            }

            return agreements;
        }
        catch ( SoapFault e )
        {
            log.error( e.getMessage() );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

}
