/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.remote;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.WsaConstants;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.apache.xmlbeans.XmlString;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.client.AgreementClient;
import org.ogf.graap.wsag.api.client.AgreementFactoryService;
import org.ogf.graap.wsag.api.client.NegotiationClient;
import org.ogf.graap.wsag.api.client.WsClient;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.schemas.graap.wsAgreement.AgreementFactoryPropertiesType;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.AgreementType;
import org.ogf.schemas.graap.wsAgreement.ContinuingFaultDocument;
import org.ogf.schemas.graap.wsAgreement.CreateAgreementInputType;
import org.ogf.schemas.graap.wsAgreement.CreateAgreementResponseDocument;
import org.ogf.schemas.graap.wsAgreement.CreatePendingAgreementInputType;
import org.ogf.schemas.graap.wsAgreement.CreatePendingAgreementResponseDocument;
import org.ogf.schemas.graap.wsAgreement.NoncriticalExtensionType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.InitiateNegotiationInputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.InitiateNegotiationInputType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.InitiateNegotiationOutputDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3c.dom.Element;

/**
 * Default implementation of an agreement factory service client.
 * 
 * @see AgreementFactoryService
 * @author Oliver Waeldrich
 * 
 */
public class RemoteAgreementFactoryServiceImpl extends WsrfResourceClient
    implements AgreementFactoryService
{

    private static final Logger LOG = Logger.getLogger( RemoteAgreementFactoryServiceImpl.class );

    /**
     * Instantiates an agreement factory service client for the given EPR.
     * 
     * @param epr
     *            the agreement factory service endpoint reference
     * 
     * @param securityProperties
     *            the security properties to use
     */
    public RemoteAgreementFactoryServiceImpl( EndpointReferenceType epr,
                                              ISecurityProperties securityProperties )
    {
        super( epr, securityProperties );
    }

    /**
     * {@inheritDoc}
     */
    public WsClient getWebServiceClient()
    {
        return this;
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient createAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnavailableException, ResourceUnknownException
    {
        try
        {
            CreateAgreementInputType request = CreateAgreementInputType.Factory.newInstance();

            if ( offer.getInitiatorEPR() != null )
            {
                request.setInitiatorAgreementEPR( offer.getInitiatorEPR() );
            }

            AgreementType agreement = request.addNewAgreementOffer();
            agreement.setName( offer.getName() );
            agreement.setContext( offer.getContext() );
            agreement.setTerms( offer.getTerms() );

            Element inputType =
                XmlUtils.createElement( WsagConstants.WSAG_CREATE_AGREEMENT_INPUT_QNAME, request.getDomNode() );
            Element body = XmlUtils.createElement( WsagConstants.WSAG_CREATE_AGREEMENT_QNAME );
            body.appendChild( body.getOwnerDocument().importNode( inputType, true ) );
            Element response = invoke( WsagConstants.WSAG_CREATE_AGREEMENT_ACTION, body );

            CreateAgreementResponseDocument respDoc =
                CreateAgreementResponseDocument.Factory.parse( response );

            AgreementClient createdAgreement =
                new RemoteAgreementClientImpl( respDoc.getCreateAgreementResponse().getCreatedAgreementEPR(),
                    getSecurityProperties().clone() );

            return createdAgreement;
        }
        catch ( SoapFault e )
        {
            handleCreateAgreementFault( e );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return createPendingAgreement( offer, null );
    }

    /**
     * {@inheritDoc}
     */
    public AgreementClient createPendingAgreement( AgreementOffer offer, EndpointReferenceType acceptanceEPR )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        try
        {
            CreatePendingAgreementInputType request = CreatePendingAgreementInputType.Factory.newInstance();

            if ( offer.getInitiatorEPR() != null )
            {
                request.setInitiatorAgreementEPR( offer.getInitiatorEPR() );
            }

            if ( acceptanceEPR != null )
            {
                request.setAgreementAcceptanceEPR( acceptanceEPR );
            }

            AgreementType agreement = request.addNewAgreementOffer();
            agreement.setName( offer.getName() );
            agreement.setContext( offer.getContext() );
            agreement.setTerms( offer.getTerms() );

            Element inputType =
                XmlUtils.createElement( WsagConstants.WSAG_CREATE_PENDING_AGREEMENT_INPUT_QNAME,
                    request.getDomNode() );
            Element body = XmlUtils.createElement( WsagConstants.WSAG_CREATE_PENDING_AGREEMENT_QNAME );
            body.appendChild( body.getOwnerDocument().importNode( inputType, true ) );
            Element response = invoke( WsagConstants.WSAG_CREATE_PENDING_AGREEMENT_ACTION, body );

            CreatePendingAgreementResponseDocument respDoc =
                CreatePendingAgreementResponseDocument.Factory.parse( response );

            AgreementClient createdAgreement =
                new RemoteAgreementClientImpl( respDoc.getCreatePendingAgreementResponse()
                                                      .getCreatedAgreementEPR(),
                    getSecurityProperties().clone() );

            return createdAgreement;
        }
        catch ( SoapFault e )
        {
            //
            // the handleFault method will throw an exception
            //
            handleCreateAgreementFault( e );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public NegotiationClient initiateNegotiation( NegotiationContextType context )
        throws NegotiationFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        return initiateNegotiation( context, WsrfResourceClient.ANONYMOUS_EPR );
    }

    /**
     * Initiates a new negotiation process with a wsag4j server.
     * 
     * @param context
     *            the negotiation context
     * @param initiatorEPR
     *            the endpoint of the wsag4j server
     * 
     * @return a new negotiation client
     * 
     * @throws NegotiationFactoryException
     *             The negotiation process was not initiated.
     * 
     * @throws ResourceUnknownException
     *             the remote resource is unknown
     * 
     * @throws ResourceUnavailableException
     *             the remote resource is unavailable
     */
    public NegotiationClient initiateNegotiation( NegotiationContextType context,
                                                  EndpointReferenceType initiatorEPR )
        throws NegotiationFactoryException, ResourceUnknownException, ResourceUnavailableException
    {

        try
        {

            InitiateNegotiationInputDocument requestDoc =
                InitiateNegotiationInputDocument.Factory.newInstance();

            InitiateNegotiationInputType request = requestDoc.addNewInitiateNegotiationInput();

            request.setNegotiationContext( context );
            request.setNoncriticalExtensionArray( new NoncriticalExtensionType[0] );
            request.setInitiatorNegotiationEPR( initiatorEPR );

            Element body = XmlUtils.createElement( WsagConstants.WSAG_INITIATE_NEGOTIATION_QNAME );
            Element inputType =
                XmlUtils.createElement( WsagConstants.WSAG_INITIATE_NEGOTIATION_INPUT_QNAME,
                    request.getDomNode() );
            body.appendChild( body.getOwnerDocument().importNode( inputType, true ) );
            Element response = invoke( WsagConstants.WSAG_INITIATE_NEGOTIATION_ACTION, body );

            InitiateNegotiationOutputDocument responseDoc =
                InitiateNegotiationOutputDocument.Factory.parse( response );

            NegotiationClient createdNegotiation =
                new RemoteNegotiationClientImpl( responseDoc.getInitiateNegotiationOutput()
                                                            .getCreatedNegotiationEPR(),
                    getSecurityProperties().clone() );

            return createdNegotiation;

        }
        catch ( SoapFault e )
        {

            LOG.error( e.getMessage() );

            Element detail = e.getDetail();

            try
            {
                // TODO we might need a more sophisticated error handling here
                XmlObject ex = XmlObject.Factory.parse( detail );
                if ( ex instanceof ContinuingFaultDocument )
                {
                    ContinuingFaultDocument result = (ContinuingFaultDocument) ex;
                    throw new NegotiationFactoryException( result.getContinuingFault().xmlText() );
                }
                if ( e.getSubCode()
                      .equals( new QName( WsaConstants.NAMESPACE_URI, "DestinationUnreachable" ) ) )
                {
                    throw new NegotiationFactoryException( new ResourceUnknownException( e.getReason() ) );
                }

                throw new ResourceUnavailableException( e );
            }
            catch ( XmlException e1 )
            {
                throw new ResourceUnavailableException( e );
            }
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public AgreementTemplateType[] getTemplates()
        throws ResourceUnavailableException, ResourceUnknownException
    {
        try
        {
            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME, WsagConstants.PREFIX
                    + ":" + "Template" );
            body.setAttribute( WsagConstants.WSAG_PREFIX_DECLARATION, WsagConstants.NAMESPACE_URI );

            Element response = invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            AgreementTemplateType[] result = null;
            AgreementFactoryPropertiesType props =
                AgreementFactoryPropertiesType.Factory.parse( response,
                    new XmlOptions().setLoadReplaceDocumentElement( null ) );
            result =
                ( props.getTemplateArray() == null ) ? new AgreementTemplateType[0]
                                : props.getTemplateArray();

            return result;
        }
        catch ( SoapFault e )
        {
            handleSoapFault( e );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public String getResourceId() throws ResourceUnknownException, ResourceUnavailableException
    {
        try
        {

            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME,
                    WsagConstants.WSDM_MUWS_PREFIX + ":" + "ResourceId" );
            body.setAttribute( WsagConstants.WSDM_MUWS_PREFIX_DECLARATION,
                WsagConstants.WSDM_MUWS_NAMESPACE_URI );

            Element response = invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            String result = XmlString.Factory.parse( response ).getStringValue();
            result = ( result == null ) ? "" : result.trim();

            return result;

        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    private void handleSoapFault( SoapFault e ) throws ResourceUnavailableException, ResourceUnknownException
    {
        LOG.error( e.getMessage() );

        //
        // TODO: We should parse for the ResourceUnknownFault document
        // in the details section
        //
        if ( e.getSubCode().equals( new QName( WsaConstants.NAMESPACE_URI, "DestinationUnreachable" ) ) )
        {
            throw new ResourceUnknownException( e.getReason() );
        }

        throw new ResourceUnavailableException( e );
    }

    private void handleCreateAgreementFault( SoapFault e )
        throws AgreementFactoryException, ResourceUnavailableException
    {
        LOG.error( e.getMessage() );

        Element detail = e.getDetail();

        try
        {
            // TODO we might need a more sophisticated error handling here
            XmlObject ex = XmlObject.Factory.parse( detail );
            if ( ex instanceof ContinuingFaultDocument )
            {
                ContinuingFaultDocument result = (ContinuingFaultDocument) ex;
                AgreementFactoryException afx =
                    new AgreementFactoryException( e.getMessage(), result.getContinuingFault() );

                Throwable cause = afx.getCause();

                while ( cause != null )
                {
                    LOG.error( cause.getMessage() );
                    cause = cause.getCause();
                }

                throw afx;
            }

            if ( e.getSubCode().equals( new QName( WsaConstants.NAMESPACE_URI, "DestinationUnreachable" ) ) )
            {
                throw new AgreementFactoryException( new ResourceUnknownException( e.getReason() ) );
            }

            throw new ResourceUnavailableException( e );

        }
        catch ( XmlException e1 )
        {
            throw new ResourceUnavailableException( e );
        }
    }

}
