/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.remote;

import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlNCName;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.client.AgreementService;
import org.ogf.graap.wsag.api.client.WsClient;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.schemas.graap.wsAgreement.AgreementContextType;
import org.ogf.schemas.graap.wsAgreement.AgreementIdDocument;
import org.ogf.schemas.graap.wsAgreement.AgreementStateDocument;
import org.ogf.schemas.graap.wsAgreement.AgreementStatePropertiesType;
import org.ogf.schemas.graap.wsAgreement.AgreementStateType;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermStateType;
import org.ogf.schemas.graap.wsAgreement.ServiceTermStateType;
import org.ogf.schemas.graap.wsAgreement.TermTreeType;
import org.ogf.schemas.graap.wsAgreement.TerminateInputType;
import org.ogf.schemas.graap.wsAgreement.TerminateResponseDocument;
import org.w3c.dom.Element;

/**
 * Default implementation of an agreement service client.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class RemoteAgreementServiceImpl implements AgreementService
{

    private WsrfResourceClient client;

    /**
     * 
     * @param client
     *            the WSRF client to use
     */
    public RemoteAgreementServiceImpl( WsrfResourceClient client )
    {
        this.client = client;
    }

    /**
     * {@inheritDoc}
     */
    public WsClient getWebServiceClient()
    {
        return client;
    }

    /**
     * {@inheritDoc}
     */
    public AgreementContextType getContext() throws ResourceUnavailableException
    {
        try
        {
            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME, "wsag:Context" );
            body.setAttribute( WsagConstants.WSAG_PREFIX_DECLARATION, WsagConstants.NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            AgreementContextType respDoc =
                AgreementContextType.Factory.parse( XmlUtils.getFirstElement( response ),
                                                    new XmlOptions().setLoadReplaceDocumentElement( null ) );

            return respDoc;
        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public GuaranteeTermStateType[] getGuaranteeTermStates() throws ResourceUnavailableException
    {
        try
        {
            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME,
                                        "wsag:GuaranteeTermState" );
            body.setAttribute( WsagConstants.WSAG_PREFIX_DECLARATION, WsagConstants.NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            AgreementStatePropertiesType props =
                AgreementStatePropertiesType.Factory.parse( response,
                                                            new XmlOptions().setLoadReplaceDocumentElement( null ) );

            GuaranteeTermStateType[] states = props.getGuaranteeTermStateArray();

            return states;

        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public String getAgreementId() throws ResourceUnavailableException
    {
        try
        {
            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME, "wsag:AgreementId" );
            body.setAttribute( WsagConstants.WSAG_PREFIX_DECLARATION, WsagConstants.NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            AgreementIdDocument id =
                AgreementIdDocument.Factory.parse( response,
                                                   new XmlOptions().setLoadReplaceDocumentElement( null ) );

            return id.getAgreementId();
        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public String getName() throws ResourceUnavailableException
    {
        try
        {
            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME, "wsag:Name" );
            body.setAttribute( WsagConstants.WSAG_PREFIX_DECLARATION, WsagConstants.NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            XmlNCName respDoc =
                XmlNCName.Factory.parse( response, new XmlOptions().setLoadReplaceDocumentElement( null ) );

            return respDoc.getStringValue();
        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public ServiceTermStateType[] getServiceTermStates() throws ResourceUnavailableException
    {
        try
        {
            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME,
                                        "wsag:ServiceTermState" );
            body.setAttribute( WsagConstants.WSAG_PREFIX_DECLARATION, WsagConstants.NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            AgreementStatePropertiesType props =
                AgreementStatePropertiesType.Factory.parse( response,
                                                            new XmlOptions().setLoadReplaceDocumentElement( null ) );

            ServiceTermStateType[] states = props.getServiceTermStateArray();

            return states;

        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public AgreementStateType getState() throws ResourceUnavailableException
    {
        try
        {
            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME, "wsag:AgreementState" );
            body.setAttribute( WsagConstants.WSAG_PREFIX_DECLARATION, WsagConstants.NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            AgreementStateDocument respDoc =
                (AgreementStateDocument) XmlObject.Factory.parse( XmlUtils.getFirstElement( response ) );

            return respDoc.getAgreementState();

        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public TermTreeType getTerms() throws ResourceUnavailableException
    {
        try
        {
            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME, "wsag:Terms" );
            body.setAttribute( WsagConstants.WSAG_PREFIX_DECLARATION, WsagConstants.NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            TermTreeType respDoc =
                TermTreeType.Factory.parse( XmlUtils.getFirstElement( response ),
                                            new XmlOptions().setLoadReplaceDocumentElement( null ) );

            return respDoc;
        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public void terminate( TerminateInputType reason ) throws ResourceUnavailableException
    {
        try
        {
            Element inputType =
                XmlUtils.createElement( WsagConstants.WSAG_TERMINATE_AGREEMENT_INPUT_QNAME,
                                        reason.getDomNode() );
            Element body = XmlUtils.createElement( WsagConstants.WSAG_TERMINATE_AGREEMENT_QNAME );
            body.appendChild( body.getOwnerDocument().importNode( inputType, true ) );
            Element response = client.invoke( WsagConstants.WSAG_TERMINATE_AGREEMENT_ACTION, body );

            TerminateResponseDocument respDoc = TerminateResponseDocument.Factory.parse( response );
            respDoc.getTerminateResponse();
        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public void destroy() throws ResourceUnknownException, ResourceUnavailableException
    {
        try
        {
            client.destroy();
        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
    }
}
