/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.remote;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.WsaConstants;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.client.NegotiationService;
import org.ogf.graap.wsag.api.client.WsClient;
import org.ogf.graap.wsag.api.exceptions.NegotiationException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.ContinuingFaultDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.AdvertiseInputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.AdvertiseInputType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.AdvertiseOutputType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.NegotiateInputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.NegotiateInputType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.NegotiateOutputType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.NegotiationPropertiesType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.TerminateInputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.TerminateResponseDocument;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Default implementation of an negotiation service client.
 * 
 * @author hrasheed
 * 
 */
public class RemoteNegotiationServiceImpl implements NegotiationService
{

    private static final Logger LOG = Logger.getLogger( RemoteNegotiationServiceImpl.class );

    private WsrfResourceClient client;

    /**
     * @param client
     *            the WSRF resource client to use
     */
    public RemoteNegotiationServiceImpl( WsrfResourceClient client )
    {
        this.client = client;
    }

    /**
     * {@inheritDoc}
     */
    public NegotiationContextType getNegotiationContext()
        throws ResourceUnknownException, ResourceUnavailableException
    {

        try
        {

            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME,
                                        WsagConstants.WSAG_NEGOTIATION_PREFIX + ":" + "NegotiationContext" );
            body.setAttribute( WsagConstants.WSAG_NEGOTIATION_PREFIX_DECLARATION,
                               WsagConstants.WSAG_NEGOTIATION_NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            NegotiationPropertiesType props =
                NegotiationPropertiesType.Factory.parse( response,
                                                         new XmlOptions().setLoadReplaceDocumentElement( null ) );
            NegotiationContextType respContext = props.getNegotiationContext();

            return respContext;

        }
        catch ( SoapFault e )
        {
            handleSoapFault( e );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public AgreementTemplateType[] getNegotiableTemplates()
        throws ResourceUnknownException, ResourceUnavailableException
    {

        try
        {

            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME,
                                        WsagConstants.WSAG_NEGOTIATION_PREFIX + ":" + "NegotiableTemplate" );
            body.setAttribute( WsagConstants.WSAG_NEGOTIATION_PREFIX_DECLARATION,
                               WsagConstants.WSAG_NEGOTIATION_NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            NegotiationPropertiesType props =
                NegotiationPropertiesType.Factory.parse( response,
                                                         new XmlOptions().setLoadReplaceDocumentElement( null ) );

            return props.getNegotiableTemplateArray();

        }
        catch ( SoapFault e )
        {
            handleSoapFault( e );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public NegotiationOfferType[] getNegotiationOffers()
        throws ResourceUnknownException, ResourceUnavailableException
    {

        try
        {

            Element body =
                XmlUtils.createElement( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_QNAME,
                                        WsagConstants.WSAG_NEGOTIATION_PREFIX + ":" + "NegotiationOffer" );
            body.setAttribute( WsagConstants.WSAG_NEGOTIATION_PREFIX_DECLARATION,
                               WsagConstants.WSAG_NEGOTIATION_NAMESPACE_URI );

            Element response = client.invoke( WsagConstants.WSRF_GET_RESOURCE_PROPERTY_ACTION, body );

            NegotiationOfferType[] offers = null;

            NegotiationPropertiesType props =
                NegotiationPropertiesType.Factory.parse( response,
                                                         new XmlOptions().setLoadReplaceDocumentElement( null ) );
            offers =
                ( props.getNegotiationOfferArray() == null ) ? new NegotiationOfferType[0]
                                : props.getNegotiationOfferArray();

            return offers;

        }
        catch ( SoapFault e )
        {
            handleSoapFault( e );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public NegotiationOfferType[] negotiate( NegotiationOfferType[] quotes )
        throws NegotiationException, ResourceUnknownException, ResourceUnavailableException
    {

        try
        {

            NegotiateInputDocument inputDoc = NegotiateInputDocument.Factory.newInstance();
            NegotiateInputType input = inputDoc.addNewNegotiateInput();
            input.setNegotiationOfferArray( quotes );

            Element body = XmlUtils.createElement( WsagConstants.WSAG_NEGOTIATION_NEGOTIATE_QNAME );
            Element inputType =
                XmlUtils.createElement( WsagConstants.WSAG_NEGOTIATION_NEGOTIATE_INPUT_QNAME,
                                        input.getDomNode() );
            body.appendChild( body.getOwnerDocument().importNode( inputType, true ) );
            Element response = client.invoke( WsagConstants.WSAG_NEGOTIATION_NEGOTIATE_ACTION, body );

            NegotiationOfferType[] counterOffers = null;

            NegotiateOutputType output =
                NegotiateOutputType.Factory.parse( response,
                                                   new XmlOptions().setLoadReplaceDocumentElement( null ) );
            counterOffers =
                ( output.getNegotiationCounterOfferArray() == null ) ? new NegotiationOfferType[0]
                                : output.getNegotiationCounterOfferArray();

            return counterOffers;

        }
        catch ( SoapFault e )
        {
            handleNegotiationFault( e );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public void advertise( NegotiationOfferType[] quotes )
        throws NegotiationException, ResourceUnknownException, ResourceUnavailableException
    {

        try
        {

            AdvertiseInputDocument inputDoc = AdvertiseInputDocument.Factory.newInstance();
            AdvertiseInputType input = inputDoc.addNewAdvertiseInput();
            input.setNegotiationOfferArray( quotes );

            Element body = XmlUtils.createElement( WsagConstants.WSAG_NEGOTIATION_ADVERTISE_QNAME );
            Element inputType =
                XmlUtils.createElement( WsagConstants.WSAG_NEGOTIATION_ADVERTISE_INPUT_QNAME,
                                        input.getDomNode() );
            body.appendChild( body.getOwnerDocument().importNode( inputType, true ) );
            Element response = client.invoke( WsagConstants.WSAG_NEGOTIATION_ADVERTISE_ACTION, body );

            AdvertiseOutputType.Factory.parse( response,
                                               new XmlOptions().setLoadReplaceDocumentElement( null ) );

        }
        catch ( SoapFault e )
        {
            handleNegotiationFault( e );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public void terminate() throws ResourceUnavailableException, ResourceUnknownException
    {

        try
        {

            Node terminateInput =
                TerminateInputDocument.Factory.newInstance().addNewTerminateInput().getDomNode();
            Element inputType =
                XmlUtils.createElement( WsagConstants.WSAG_NEGOTIATION_TERMINATE_INPUT_QNAME, terminateInput );
            Element body = XmlUtils.createElement( WsagConstants.WSAG_NEGOTIATION_TERMINATE_QNAME );
            body.appendChild( body.getOwnerDocument().importNode( inputType, true ) );
            Element response = client.invoke( WsagConstants.WSAG_NEGOTIATION_TERMINATE_ACTION, body );

            TerminateResponseDocument respDoc = TerminateResponseDocument.Factory.parse( response );
            respDoc.getTerminateResponse();

        }
        catch ( SoapFault e )
        {
            handleSoapFault( e );
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public WsClient getWebServiceClient()
    {
        return client;
    }

    /**
     * {@inheritDoc}
     */
    public void destroy() throws ResourceUnavailableException, ResourceUnknownException
    {
        try
        {
            client.destroy();
        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    private void handleNegotiationFault( SoapFault e )
        throws NegotiationException, ResourceUnavailableException
    {

        LOG.error( e.getMessage() );

        Element detail = e.getDetail();

        try
        {
            // TODO we might need a more sophisticated error handling here
            XmlObject ex = XmlObject.Factory.parse( detail );
            if ( ex instanceof ContinuingFaultDocument )
            {
                ContinuingFaultDocument result = (ContinuingFaultDocument) ex;
                throw new NegotiationException( result.getContinuingFault().xmlText() );
            }

            if ( e.getSubCode().equals( new QName( WsaConstants.NAMESPACE_URI, "DestinationUnreachable" ) ) )
            {
                throw new NegotiationException( new ResourceUnknownException( e.getReason() ) );
            }

            throw new ResourceUnavailableException( e );

        }
        catch ( XmlException e1 )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    private void handleSoapFault( SoapFault e ) throws ResourceUnavailableException, ResourceUnknownException
    {

        LOG.error( e.getMessage() );
        //
        // TODO: We should parse for the ResourceUnknownFault document in the details section
        //
        if ( e.getSubCode().equals( new QName( WsaConstants.NAMESPACE_URI, "DestinationUnreachable" ) ) )
        {
            throw new ResourceUnknownException( e.getReason() );
        }

        throw new ResourceUnavailableException( e );
    }

}
