/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.client.remote;

import java.util.Date;
import java.util.Properties;

import javax.security.auth.x500.X500Principal;
import javax.xml.namespace.QName;

import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.WsaConstants;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.resource.properties.set.SetRequest;
import org.apache.muse.ws.resource.remote.WsResourceClient;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;
import org.ogf.graap.wsag.api.client.WsClient;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.graap.wsag.security.core.SecurityConstants;
import org.ogf.graap.wsag4j.types.engine.ServerIdentityDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * AbstractClient
 * 
 * @author Oliver Waeldrich
 * 
 */
public class WsrfResourceClient implements WsClient
{

    /**
     * Definition of an anonymous SOAP sender EPR.
     */
    public static final EndpointReferenceType ANONYMOUS_EPR =
        convertMuseEndpoint( WsaConstants.ANONYMOUS_EPR );

    /**
     * Key to lookup extra header fields.
     */
    @Deprecated
    public static final String EXTRA_HEADERS = "http://de.fraunhofer.scai.wsag4j/extra-headers";

    //
    // the MUSE WS client, all requests are delegated to that client
    //
    private WsResourceClient client;

    //
    // the SOAP client, it sends the SOAP message
    //
    private Axis2SoapClient soapClient;

    //
    // the login context used to create this client
    //
    private ISecurityProperties securityProperties;

    /**
     * The default SOAP client used for network communication.
     */
    private static Axis2SoapClient defaultSOAPClient = null;

    /**
     * @return the defaultSOAPClient
     */
    public static Axis2SoapClient getDefaultSOAPClient()
    {
        return defaultSOAPClient;
    }

    /**
     * @param defaultSOAPClient
     *            the default SOAP client to set
     */
    public static void setDefaultSOAPClient( Axis2SoapClient defaultSOAPClient )
    {
        WsrfResourceClient.defaultSOAPClient = defaultSOAPClient;
    }

    /**
     * Specifies whether the default client should be used or not (default: false).
     */
    private static boolean useDefaultSOAPClient = false;

    /**
     * @return the useDefaultSOAPClient
     */
    public static boolean isUseDefaultSOAPClient()
    {
        return useDefaultSOAPClient;
    }

    /**
     * @param useDefaultSOAPClient
     *            the useDefaultSOAPClient to set
     */
    public static void setUseDefaultSOAPClient( boolean useDefaultSOAPClient )
    {
        WsrfResourceClient.useDefaultSOAPClient = useDefaultSOAPClient;
    }

    //
    // private constructors
    //
    private WsrfResourceClient( EndpointReferenceType destination, EndpointReferenceType source,
                                ISecurityProperties securityProperties )
    {
        if ( ( useDefaultSOAPClient ) && ( defaultSOAPClient != null ) )
        {
            soapClient = defaultSOAPClient;
        }
        else
        {
            soapClient = new Axis2SoapClient( securityProperties );
        }

        client = new WsResourceClient( convertEndpoint( destination ), convertEndpoint( source ), soapClient );
    }

    /**
     * Creates a WSRFResourceClient with the given endpoint and the provided security properties.
     * 
     * @param destination
     *            the WSRF resource endpoint
     * 
     * @param securityProperties
     *            the security properties to use
     * 
     */
    public WsrfResourceClient( EndpointReferenceType destination, ISecurityProperties securityProperties )
    {
        this( destination, ANONYMOUS_EPR, securityProperties );
        this.securityProperties = securityProperties;

        X500Principal serverIdentity = extractServerIdentity( destination );
        if ( serverIdentity != null )
        {
            this.securityProperties.getProperties().put( SecurityConstants.X500_SERVER_IDENTITY,
                                                         serverIdentity );
        }
    }

    private X500Principal extractServerIdentity( EndpointReferenceType epr )
    {
        X500Principal principal = null;

        if ( epr.isSetMetadata() )
        {
            XmlObject[] identity =
                epr.getMetadata().selectChildren( ServerIdentityDocument.type.getDocumentElementName() );
            if ( identity.length > 0 )
            {
                XmlString serverId = (XmlString) identity[0];
                principal = new X500Principal( serverId.getStringValue() );
            }
        }

        return principal;
    }

    private static EndpointReferenceType convertMuseEndpoint( EndpointReference epr )
    {
        try
        {
            EndpointReferenceDocument converted =
                (EndpointReferenceDocument) XmlObject.Factory.parse( epr.toXML() );
            return converted.getEndpointReference();
        }
        catch ( XmlException e )
        {
            String message = "Error while converting MUSE EPR to endpoint reference.";
            throw new RuntimeException( message, e );
        }
    }

    private static EndpointReference convertEndpoint( EndpointReferenceType epr )
    {
        try
        {
            EndpointReferenceDocument doc = EndpointReferenceDocument.Factory.newInstance();
            doc.setEndpointReference( epr );
            return new EndpointReference( (Element) doc.getDomNode().getFirstChild() );
        }
        catch ( SoapFault e )
        {
            String message = "Error while converting endpoint reference to MUSE EPR.";
            throw new RuntimeException( message, e );
        }
    }

    /**
     * @return Returns the properties.
     */
    public Properties getProperties()
    {
        return soapClient.getProperties();
    }

    /**
     * @param properties
     *            The properties to set.
     */
    public void setProperties( Properties properties )
    {
        soapClient.setProperties( properties );
    }

    /**
     * @return the securityProperties
     */
    public ISecurityProperties getSecurityProperties()
    {
        return securityProperties;
    }

    /**
     * @return Returns the endpoint reference for the service.
     */
    public EndpointReferenceType getEndpoint()
    {
        return convertMuseEndpoint( client.getEndpointReference() );
    }

    /**
     * @param qname
     *            the name of the resource property to delete
     * @throws SoapFault
     *             indicates an error that occurred while deleting the resource property
     * @see org.apache.muse.ws.resource.remote.WsResourceClient#deleteResourceProperty(javax.xml.namespace.QName)
     */
    public void deleteResourceProperty( QName qname ) throws SoapFault
    {
        client.deleteResourceProperty( qname );
    }

    /**
     * @throws SoapFault
     *             indicates an error while destroying the WS resource
     * 
     * @see org.apache.muse.ws.resource.remote.WsResourceClient#destroy()
     */
    public void destroy() throws SoapFault
    {
        client.destroy();
    }

    /**
     * Retrieves multiple resource properties from a WSRF resource.
     * 
     * @param qnames
     *            the QNames of the resource properties
     * 
     * @return the retrieved resource properties
     * 
     * @throws SoapFault
     *             indicates an error while retrieving the resource properties
     * 
     * @see org.apache.muse.ws.resource.remote.WsResourceClient#getMultipleResourceProperties(javax.xml.namespace.QName[])
     */
    public Element[] getMultipleResourceProperties( QName[] qnames ) throws SoapFault
    {
        return client.getMultipleResourceProperties( qnames );
    }

    /**
     * Retrieves a single resource property from a WSRF resource.
     * 
     * @param qname
     *            the QName of the resource property
     * 
     * @return the retrieved resource property
     * 
     * @throws SoapFault
     *             indicates an error while retrieving the resource property
     * 
     * @see org.apache.muse.ws.resource.remote.WsResourceClient#getResourceProperty(javax.xml.namespace.QName)
     */
    public Element[] getResourceProperty( QName qname ) throws SoapFault
    {
        return client.getResourceProperty( qname );
    }

    /**
     * Retrieves a resource property document from a WSRF resource
     * 
     * @return the resource property document
     * 
     * @throws SoapFault
     *             indicates an error while retrieving the resource properties document
     * 
     * @see org.apache.muse.ws.resource.remote.WsResourceClient#getResourcePropertyDocument()
     */
    public Element getResourcePropertyDocument() throws SoapFault
    {
        return client.getResourcePropertyDocument();
    }

    /**
     * @return if the client trace the messages or not
     * 
     * @see WsrfResourceClient#isUsingTrace()
     */
    public boolean isUsingTrace()
    {
        return client.isUsingTrace();
    }

    /**
     * Queries multiple resource properties from a WSRF resource.
     * 
     * @param query
     *            the query
     * 
     * @param dialect
     *            the query dialect (e.g. XPath)
     * 
     * @return the retrieved resource properties
     * 
     * @throws SoapFault
     *             indicates an error while querying the resource properties
     * 
     * @see org.apache.muse.ws.resource.remote.WsResourceClient#queryResourceProperties(java.lang.String,
     *      java.lang.String)
     */
    public Node[] queryResourceProperties( String query, String dialect ) throws SoapFault
    {
        return client.queryResourceProperties( query, dialect );
    }

    /**
     * Sets a resource property.
     * 
     * @param request
     *            the request object
     * @throws SoapFault
     *             indicates an error while setting the resource property
     * @see org.apache.muse.ws.resource.remote.WsResourceClient#setResourceProperties(org.apache.muse.ws.resource.properties.set.SetRequest)
     */
    public void setResourceProperties( SetRequest request ) throws SoapFault
    {
        client.setResourceProperties( request );
    }

    /**
     * Sets the termination time for a WSRF resource.
     * 
     * @param time
     *            the termination time
     * 
     * @return the scheduled termination time
     * 
     * @throws SoapFault
     *             indicates an error while setting the termination time
     * 
     * @see org.apache.muse.ws.resource.remote.WsResourceClient#setTerminationTime(java.util.Date)
     */
    public Date setTerminationTime( Date time ) throws SoapFault
    {
        return client.setTerminationTime( time );
    }

    /**
     * @param trace
     *            toggles the trace capabilities
     * 
     * @see WsrfResourceClient#setTrace(boolean)
     */
    public void setTrace( boolean trace )
    {
        client.setTrace( trace );
    }

    /**
     * Updates a resource property.
     * 
     * @param qname
     *            the name of the resource property to update
     * @param values
     *            the value(s) of the property
     * @throws SoapFault
     *             indicates an error while updating the resource property
     * 
     * @see org.apache.muse.ws.resource.remote.WsResourceClient#updateResourceProperty(javax.xml.namespace.QName,
     *      java.lang.Object[])
     */
    public void updateResourceProperty( QName qname, Object[] values ) throws SoapFault
    {
        client.updateResourceProperty( qname, values );
    }

    /**
     * Invokes an action on a WSRF resource
     * 
     * @param action
     *            the WSA action
     * 
     * @param soapBody
     *            the SOAP body to send
     * 
     * @return the SOAP response
     * 
     * @throws SoapFault
     *             indicates an error while invoking the SOAP client
     * 
     * @see org.apache.muse.core.AbstractResourceClient#invoke(java.lang.String, org.w3c.dom.Element)
     */
    public Element invoke( String action, Element soapBody ) throws SoapFault
    {
        return client.invoke( action, soapBody );
    }

}
